@@ -4,6 +4,7 @@ import typing as t
 
 import logging
 
+from contextlib import asynccontextmanager
 from contextvars import ContextVar
 from urllib.parse import urlsplit
 
@@ -32,7 +33,7 @@ class Database:
         self.url = url
         self.logger = logger
         self.backend: ABCDatabaseBackend = backend_cls(parsed_url, logger=self.logger, **options)
-        self._conn_ctx: ContextVar = ContextVar('connection')
+        self._conn_ctx: ContextVar = ContextVar('connection', default=None)
 
     async def connect(self) -> Database:
         """Connect the database."""
@@ -60,44 +61,71 @@ class Database:
 
     __aexit__ = disconnect
 
+    def __create_connection__(self) -> ABCConnection:
+        """Create a connection."""
+        conn = self.backend.connection()
+        self._conn_ctx.set(conn)
+        return conn
+
+    def __get_connection__(self, force_create: bool = True) -> t.Tuple[ABCConnection, bool]:
+        """Get/create a connection from/to the current context."""
+        created = force_create
+        conn = self._conn_ctx.get()
+        if conn is None or force_create:
+            created = True
+            conn = self.__create_connection__()
+        return conn, created
+
     def connection(self, create: bool = True) -> ABCConnection:
         """Get/create a connection from/to the current context."""
-        conn = self._conn_ctx.get(None)
-        if conn is None or create:
-            conn = self.backend.connection()
-            self._conn_ctx.set(conn)
-
+        conn, _ = self.__get_connection__(create)
         return conn
 
     def transaction(self, create: bool = False) -> ABCTransaction:
+        """Create a transaction."""
         return self.connection(create).transaction()
 
     async def execute(self, query: t.Any, *params, **options) -> t.Any:
-        conn = await self.connection(False).acquire()
-        return await conn.execute(query, *params)
+        """Execute a query."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.execute(query, *params)
 
     async def executemany(self, query: t.Any, *params, **options) -> t.Any:
-        conn = await self.connection(False).acquire()
-        return await conn.executemany(query, *params, **options)
+        """Execute a query many times."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.executemany(query, *params, **options)
 
     async def fetchall(self, query: t.Any, *params, **options) -> t.List[t.Mapping]:
-        conn = await self.connection(False).acquire()
-        return await conn.fetchall(query, *params, **options)
+        """Fetch all rows."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.fetchall(query, *params, **options)
 
     async def fetchmany(self, size: int, query: t.Any, *params, **options) -> t.List[t.Mapping]:
-        conn = await self.connection(False).acquire()
-        return await conn.fetchmany(size, query, *params, **options)
+        """Fetch rows."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.fetchmany(size, query, *params, **options)
 
     async def fetchone(self, query: t.Any, *params, **options) -> t.Optional[t.Mapping]:
-        conn = await self.connection(False).acquire()
-        return await conn.fetchone(query, *params)
+        """Fetch a row."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.fetchone(query, *params)
 
     async def fetchval(self, query: t.Any, *params, column: t.Any = 0, **options) -> t.Any:
-        conn = await self.connection(False).acquire()
-        return await conn.fetchval(query, *params, column=column, **options)
+        """Fetch a value."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            return await conn.fetchval(query, *params, column=column, **options)
 
     async def iterate(self, query: t.Any, *params, **options) -> t.AsyncIterator:
-        """Iterate through rows."""
-        conn = await self.connection(False).acquire()
-        async for res in conn.iterate(query, *params, **options):
-            yield res
+        """Iterate through results."""
+        async with manage_connection(*self.__get_connection__(False)) as conn:
+            async for res in conn.iterate(query, *params, **options):
+                yield res
+
+
+@asynccontextmanager
+async def manage_connection(connection: ABCConnection, release: bool = True):
+    """Acquire/release connections."""
+    await connection.acquire()
+    yield connection
+    if release:
+        await connection.release()
